<div xmlns="http://www.w3.org/1999/xhtml" xmlns:py="http://genshi.edgewall.org/"
    class="${css_class}"
    py:attrs='value_of("attrs", "")'
>
<a href="${controller}${entity.__name__}/">${entity.__name__}</a>
</div>