<tr xmlns="http://www.w3.org/1999/xhtml" xmlns:py="http://genshi.edgewall.org/"
    class="${css_class}"
    py:attrs='value_of("attrs", "")'
    >
    <td>
        <b>$field_name</b>
    </td>
    <td> ${value[field_name]}
    </td>
</tr>