<div xmlns="http://www.w3.org/1999/xhtml"
       xmlns:py="http://genshi.edgewall.org/">
<table
       id="${id}" class="${css_class}">
    <thead py:if="columns">
        <tr>
            <th py:if="pks"/>
            <th py:for="i, col in enumerate(columns)" class="col_${i}">
            ${col.title}
            </th>
        </tr>
    </thead>
    <tbody>
        <tr py:for="i, row in enumerate(value)"
            class="${i%2 and 'odd' or 'even'}">
            <td py:if="pks">
<?python

pklist = ''
for pk in pks:
    pklist+=str(row[pk])
    pklist+='/'
?>
            <a href="${controller}${pklist}">edit</a> |
            <a href="${controller}${pklist}delete">delete</a>
            </td>
            <td py:for="col in columns">${col.get_field(row, displays_on='genshi')}</td>
        </tr>
    </tbody>
</table>
    <py:if test="len(value) == 0">
      No Records Found.
    </py:if>
</div>